"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirebaseFunction = void 0;
const functions = __importStar(require("firebase-functions"));
const parameter_1 = require("../parameter");
const logger_1 = require("../logger");
const messageAuthenticator_1 = require("../messageAuthenticator");
const bytesCoder_1 = require("../bytesCoder");
const utils_1 = require("../utils");
var FirebaseFunction;
(function (FirebaseFunction_1) {
    let Flattable;
    (function (Flattable) {
        function flatten(value) {
            if (typeof value === 'boolean' || typeof value === 'string' || typeof value === 'number' || value === null)
                return value;
            if (Array.isArray(value))
                return value.map(element => flatten(element));
            if (typeof value === 'object') {
                if ('flatten' in value)
                    return value.flatten;
                return (0, utils_1.mapRecord)(value, element => flatten(element));
            }
            return null;
        }
        Flattable.flatten = flatten;
    })(Flattable = FirebaseFunction_1.Flattable || (FirebaseFunction_1.Flattable = {}));
    function verifyMacTag(tag, parameters, key) {
        const messageAuthenticater = new messageAuthenticator_1.HMAC(key);
        const macTagByteCoder = new bytesCoder_1.HexBytesCoder();
        const rawMacTag = macTagByteCoder.encode(tag);
        const parametersBytesCoder = new bytesCoder_1.Utf8BytesCoder();
        const encodedParameters = parametersBytesCoder.encode(JSON.stringify(parameters));
        return messageAuthenticater.verify(encodedParameters, rawMacTag);
    }
    FirebaseFunction_1.verifyMacTag = verifyMacTag;
    function convertToHttpsError(error) {
        if (error instanceof functions.https.HttpsError)
            return error;
        return new functions.https.HttpsError('unknown', 'Unknown error occured', error);
    }
    async function execute(_function) {
        try {
            return await _function()
                .then(value => utils_1.Result.success(value))
                .catch(error => utils_1.Result.failure(convertToHttpsError(error)));
        }
        catch (error) {
            return utils_1.Result.failure(convertToHttpsError(error));
        }
    }
    FirebaseFunction_1.execute = execute;
    function create(
    // eslint-disable-next-line @typescript-eslint/naming-convention
    FirebaseFunction, macKey, regions = []) {
        return functions.region(...regions).https.onCall(async (data, context) => {
            const result = await execute(async () => {
                const verboseLogger = 'verboseLogger' in data && data.verboseLogger === true;
                const logger = logger_1.Logger.start('FirebaseFunction.create', null, 'info', verboseLogger);
                const verified = verifyMacTag(data.macTag, data.parameters, macKey);
                if (!verified)
                    throw new functions.https.HttpsError('permission-denied', 'Invalid MAC tag');
                const parameterContainer = new parameter_1.ParameterContainer(data.parameters, logger.nextIndent);
                const firebaseFunction = new FirebaseFunction(parameterContainer, context.auth ?? null, logger.nextIndent);
                const returnValue = await firebaseFunction.execute();
                return Flattable.flatten(returnValue);
            });
            return result;
        });
    }
    FirebaseFunction_1.create = create;
})(FirebaseFunction = exports.FirebaseFunction || (exports.FirebaseFunction = {}));
