"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArrayParameterBuilder = void 0;
const utils_1 = require("../../utils");
class ArrayParameterBuilder {
    builder;
    length;
    constructor(builder, length = null) {
        this.builder = builder;
        this.length = length;
    }
    get expectedTypes() {
        return new Set(['object']);
    }
    build(value, logger) {
        logger.log('ArrayParameterBuilder.build', { expectedTypes: this.builder.expectedTypes, value: value, length: this.length });
        if (value === null || !Array.isArray(value))
            throw new utils_1.HttpsError('invalid-argument', 'Value is not an array.', logger);
        if (this.length !== null && value.length !== this.length)
            throw new utils_1.HttpsError('invalid-argument', `Value array has not the expectd length ${length}.`, logger);
        return value.map((element, index) => {
            if (!this.builder.expectedTypes.has(typeof element))
                throw new utils_1.HttpsError('invalid-argument', `Array element ${index} has an invalid type: ${typeof element}.`, logger);
            return this.builder.build(element, logger.nextIndent);
        });
    }
}
exports.ArrayParameterBuilder = ArrayParameterBuilder;
