"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const src_1 = require("../../src");
const testSrc_1 = require("../../testSrc");
describe('utcDate', () => {
    it('timezone', () => {
        const date = new src_1.UtcDate(2023, 1, 1, 12, 0, 'Europe/Berlin');
        (0, testSrc_1.expect)(date).to.be.deep.equal(new src_1.UtcDate(2023, 1, 1, 12, 0));
    });
    it('from date', () => {
        (0, testSrc_1.expect)(src_1.UtcDate.fromIsoDate(src_1.UtcDate.now.toIsoDate).day).to.be.equal(new Date().getDate());
        const date = src_1.UtcDate.fromDate(new Date('2023-02-07T12:34:56+01:00'));
        (0, testSrc_1.expect)(date).to.be.deep.equal(new src_1.UtcDate(2023, 2, 7, 11, 34));
    });
    it('encode', () => {
        (0, testSrc_1.expect)(new src_1.UtcDate(2024, 4, 3, 9, 3).encoded).to.be.equal('2024-04-03-09-03');
        (0, testSrc_1.expect)(new src_1.UtcDate(2024, 11, 23, 19, 21).encoded).to.be.equal('2024-11-23-19-21');
    });
    it('encode decode', () => {
        const date = src_1.UtcDate.fromDate(new Date('2023-02-07T12:34:56+01:00'));
        const { encoded } = date;
        (0, testSrc_1.expect)(encoded).to.be.equal('2023-02-07-11-34');
        (0, testSrc_1.expect)(src_1.UtcDate.decode(encoded)).to.be.deep.equal(date);
        (0, testSrc_1.expect)(src_1.UtcDate.decode('')).to.be.deep.equal(new src_1.UtcDate(0, 0, 0, 0, 0));
    });
    it('setted', () => {
        const date = src_1.UtcDate.fromDate(new Date('2023-02-07T12:34:56+01:00'));
        let newDate = date.setted({ year: 2022, month: 5, day: 1 });
        (0, testSrc_1.expect)(newDate).to.be.deep.equal(new src_1.UtcDate(2022, 5, 1, 11, 34));
        newDate = date.setted({ hour: 0, minute: 0 });
        (0, testSrc_1.expect)(newDate).to.be.deep.equal(new src_1.UtcDate(2023, 2, 7, 0, 0));
        newDate = date.setted({ month: 1, day: 40, minute: 65 });
        (0, testSrc_1.expect)(newDate).to.be.deep.equal(new src_1.UtcDate(2023, 2, 9, 12, 5));
    });
    it('advanced', () => {
        const date = src_1.UtcDate.fromDate(new Date('2023-02-07T12:34:56+01:00'));
        let newDate = date.advanced({ year: 1, month: 1, day: 1 });
        (0, testSrc_1.expect)(newDate).to.be.deep.equal(new src_1.UtcDate(2024, 3, 8, 11, 34));
        newDate = date.advanced({ hour: 4, minute: 2 });
        (0, testSrc_1.expect)(newDate).to.be.deep.equal(new src_1.UtcDate(2023, 2, 7, 15, 36));
        newDate = date.advanced({ month: 1, day: 30, minute: 30 });
        (0, testSrc_1.expect)(newDate).to.be.deep.equal(new src_1.UtcDate(2023, 4, 6, 12, 4));
    });
    it('compare', () => {
        const date1 = src_1.UtcDate.fromDate(new Date('2024-02-07T12:34:56+01:00'));
        const date2 = src_1.UtcDate.fromDate(new Date('2023-02-08T12:34:56+01:00'));
        const date3 = src_1.UtcDate.fromDate(new Date('2023-02-07T12:35:56+01:00'));
        const date4 = src_1.UtcDate.fromDate(new Date('2023-02-07T12:34:56+01:00'));
        const date5 = src_1.UtcDate.fromDate(new Date('2023-02-07T13:34:56+01:00'));
        const date6 = src_1.UtcDate.fromDate(new Date('2023-03-07T13:34:56+01:00'));
        (0, testSrc_1.expect)(date1.compare(date2)).to.be.equal('greater');
        (0, testSrc_1.expect)(date1.compare(date3)).to.be.equal('greater');
        (0, testSrc_1.expect)(date1.compare(date4)).to.be.equal('greater');
        (0, testSrc_1.expect)(date2.compare(date3)).to.be.equal('greater');
        (0, testSrc_1.expect)(date2.compare(date4)).to.be.equal('greater');
        (0, testSrc_1.expect)(date3.compare(date4)).to.be.equal('greater');
        (0, testSrc_1.expect)(date1.compare(date1)).to.be.equal('equal');
        (0, testSrc_1.expect)(date2.compare(date1)).to.be.equal('less');
        (0, testSrc_1.expect)(date3.compare(date1)).to.be.equal('less');
        (0, testSrc_1.expect)(date4.compare(date1)).to.be.equal('less');
        (0, testSrc_1.expect)(date3.compare(date2)).to.be.equal('less');
        (0, testSrc_1.expect)(date4.compare(date2)).to.be.equal('less');
        (0, testSrc_1.expect)(date4.compare(date3)).to.be.equal('less');
        (0, testSrc_1.expect)(date4.compare(date5)).to.be.equal('less');
        (0, testSrc_1.expect)(date5.compare(date4)).to.be.equal('greater');
        (0, testSrc_1.expect)(date5.compare(date6)).to.be.equal('less');
        (0, testSrc_1.expect)(date6.compare(date5)).to.be.equal('greater');
    });
});
