"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirebaseFunctionsCaller = void 0;
const functions_1 = require("firebase/functions");
const src_1 = require("../src");
const axios_1 = __importDefault(require("axios"));
class FirebaseFunctionsCaller {
    functions;
    options;
    name;
    constructor(functions, options, name = '') {
        this.functions = functions;
        this.options = options;
        this.name = name;
    }
    function(key) {
        const name = this.name === '' ? `${this.name}-${key}` : key;
        return new FirebaseFunctionsCaller(this.functions[key], this.options, name);
    }
    createMacTag(parameters, key) {
        const messageAuthenticater = new src_1.HMAC(key);
        const parametersBytesCoder = new src_1.Utf8BytesCoder();
        const encodedParameters = parametersBytesCoder.encode(JSON.stringify(parameters));
        const rawTag = messageAuthenticater.sign(encodedParameters);
        const macTagByteCoder = new src_1.HexBytesCoder();
        return macTagByteCoder.decode(rawTag);
    }
    get functionsInstance() {
        const functionsInstance = (0, functions_1.getFunctions)(this.options.app, this.options.region);
        if (this.options.useEmulator)
            (0, functions_1.connectFunctionsEmulator)(functionsInstance, '127.0.0.1', 5001);
        return functionsInstance;
    }
    async callFunction(parameters) {
        const macTag = this.createMacTag(parameters, this.options.macKey);
        const callableFunction = (0, functions_1.httpsCallable)(this.functionsInstance, this.name);
        const response = await callableFunction({
            verboseLogger: true,
            macTag: macTag,
            parameters: parameters
        });
        const result = src_1.Result.from(response.data);
        return result.get();
    }
    async callRequest(parameters) {
        const macTag = this.createMacTag(parameters, this.options.macKey);
        const url = `${this.options.requestBaseUrl}/${this.options.region}/${this.name}`;
        const response = await axios_1.default.post(url, {
            verboseLogger: true,
            macTag: macTag,
            parameters: parameters
        });
        const result = src_1.Result.from(response.data);
        return result.get();
    }
}
exports.FirebaseFunctionsCaller = FirebaseFunctionsCaller;
